#include <stdio.h>
#include <math.h>
#include "../../asl.h"
#include "../sip.h"
#include "int.h"


/************************************************************
Evaluate the integral by a gaussian formula
gt     - pointer to function
nt     - number of infinite variables
nx     - number of finite variables
lb     - array of lower bounds
ub     - array of upper bounds
x      - finite variables
t      - infinite variables (null initialized)
j      - number of infinite constraint
degree - number of points
grad   - gradient
dlevel - derivative level

return the integral value
************************************************************/
void gaussian(double (*gt) (), double *lb, double *ub, int nx, double *x,
	      int nt, double *t, int j, int degree, double *integral,
	      double *isgn, int dlevel, double *grad, double miu)
{
  int i, k;
  double factor, ival, isgnval;

  double GAUSSP6[6]={-0.2386192, -0.6612094, -0.9324695,
		      0.2386192,  0.6612094,  0.9324695};
  double GAUSSC6[6]={0.4679139, 0.3607616, 0.1713245,
		     0.4679139, 0.3607616, 0.1713245};
    
  double GAUSSP8[8]={
    0.9602898565,   0.7966664774,  0.5255324099,  0.1834346425,
    -0.9602898565, -0.7966664774, -0.5255324099, -0.1834346425};
  double GAUSSC8[8]={0.1012285363, 0.223810345, 0.3137066459, 0.3626837834,
		     0.1012285363, 0.223810345, 0.3137066459, 0.3626837834};
  
  double GAUSSP16[16]={0.98940093, 0.94457502, 0.86563120, 0.75540441,
		       0.61787624, 0.45801678, 0.28160355, 0.09501251,
		       -0.98940093, -0.94457502, -0.86563120, -0.75540441,
		       -0.61787624, -0.45801678, -0.28160355, -0.09501251};
  double GAUSSC16[16]={0.02715246, 0.06225352, 0.09515851, 0.12462897,
		       0.14959599, 0.16915652, 0.18260342, 0.18945061,
		       0.02715246, 0.06225352, 0.09515851, 0.12462897,
		       0.14959599, 0.16915652, 0.18260342, 0.18945061};

#if defined __linux__ || defined linux
  double gd[nx];
#else
  double *gd;
  gd=mymalloc(nx*sizeof(double));
#endif

  
  
  *integral=0;
  *isgn=0;
  if(dlevel==FIRST_DERIV)
    memset(grad, 0, sizeof(double)*nx);

  switch(degree){
  case 6:
  case 8:
  case 16:
    break;
  default:
    printf("Invalid number of points in gaussian formula\n");
    printf("Using 16 points\n");
    degree=16;
  }

  
  for(i=0;i<degree;i++){
    factor=(ub[0]-lb[0])/2;
    switch(degree){
    case 6:
      t[0]=factor*GAUSSP6[i]+(ub[0]+lb[0])/2;
      break;
    case 8:
      t[0]=factor*GAUSSP8[i]+(ub[0]+lb[0])/2;
      break;
    case 16:
      t[0]=factor*GAUSSP16[i]+(ub[0]+lb[0])/2;
      break;
    }
    /* constraints <= UB*/
    ival=gt(x, t, j, dlevel, gd, miu)*factor;
    if(ival>0){
      isgnval=factor;
    } else {
      isgnval=0;
    }
    switch(degree){
    case 6:
      *integral+=GAUSSC6[i]*ival;
      *isgn+=GAUSSC6[i]*isgnval;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=GAUSSC6[i]*(gd[k]);
      }
      break;
    case 8:
      *integral+=GAUSSC8[i]*ival;
      *isgn+=GAUSSC8[i]*isgnval;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=GAUSSC8[i]*(gd[k]);
      }
      break;
    case 16:
      *integral+=GAUSSC16[i]*ival;
      *isgn+=GAUSSC16[i]*isgnval;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=GAUSSC16[i]*(gd[k]);
      }
      break;
    }
  }

#if !defined __linux__ && !defined linux
  free(gd);
#endif
  /*  printf("Gauss %.12f\n", *integral);*/
}



/*******************************************************************
Evaluate the integral by a gaussian adaptative formula
gt     - pointer to function
nt     - number of infinite variables
nx     - number of finite variables
lb     - array of lower bounds
ub     - array of upper bounds
x      - finite variables
t      - infinite variables (null initialized)
degree - number of start points in the formula
j      - number of infinite constraint
error  - error between refinements
integral- the integral value
isgn   - the sign integral value
dlevel - derivative level
grad   - gradient
************************************************************/
void adapt_gauss(double (*gt) (), double *lb, double *ub, int nx, double *x,
		 int nt, double *t, int degree, int j, double *integral,
		 double *isgn, double error, double amp, int dlevel,
		 double *grad, double miu)
{
  double mid[1];
  double ival, isgnval, ival2, isgnval2, ival3, isgnval3;
  int k;

#if defined __linux__ || defined linux
  double gd[nx], gd2[nx], gd3[nx];
#else
  double *gd, *gd2, *gd3;
  gd=mymalloc(nx*sizeof(double));
  gd2=mymalloc(nx*sizeof(double));
  gd3=mymalloc(nx*sizeof(double));
#endif
  
  
  mid[0]=(lb[0]+ub[0])/2;
  
  gaussian(gt, lb, ub, nx, x, nt, t, j, degree, &ival, &isgnval, dlevel, gd,
	   miu);
  
  gaussian(gt, lb, mid, nx, x, nt, t, j, degree, &ival2, &isgnval2,
	   dlevel, gd2, miu);
  gaussian(gt, mid, ub, nx, x, nt, t, j, degree, &ival3, &isgnval3,
	   dlevel, gd3, miu);
  
  ival2+=ival3; isgnval2+=isgnval3;
  
  if((ub[0]-lb[0])<=amp || fabs(ival-ival2)<error){
    *integral=ival2;
    *isgn=isgnval2;
    for(k=0;k<nx;k++)
      grad[k]=gd2[k]+gd3[k];
    printf("%f,%f,",lb[0],mid[0]);
#if !defined __linux__ && !defined linux
    free(gd);
    free(gd2);
    free(gd3);
#endif
    return;
  }

  adapt_gauss(gt, lb, mid, nx, x, nt, t, degree, j, &ival2, &isgnval2,
	      error, amp, dlevel, gd2, miu);
  adapt_gauss(gt, mid, ub, nx, x, nt, t, degree, j, &ival3, &isgnval3,
	      error, amp, dlevel, gd3, miu);

  *integral=ival2+ival3;
  *isgn=isgnval2+isgnval3;
  for(k=0;k<nx;k++)
    grad[k]=gd2[k]+gd3[k];

#if !defined __linux__ && !defined linux
    free(gd);
    free(gd2);
    free(gd3);
#endif

  return;
}




/************************************************************
Evaluate the integral by a Newton-Cotes trapezoid formula
gt     - pointer to function
nt     - number of infinite variables
nx     - number of finite variables
lb     - array of lower bounds
ub     - array of upper bounds
x      - finite variables
t      - infinite variables (null initialized)
degree - number of points in the formula
j      - number of infinite constraint

return the integral value
************************************************************/
void trapezoid(double (*gt) (), double *lb, double *ub, int nx, double *x,
	       int nt, double *t, int degree, int j, double *integral,
	       double *isgn, int dlevel, double *grad, double miu)
{
  int i, k;
  double ival, isgnval, space;

#if defined __linux__ || defined linux
  double gd[nx];
#else
  double *gd;
  gd=mymalloc(nx*sizeof(double));
#endif


  *integral=0;
  *isgn=0;
  if(dlevel==FIRST_DERIV)
    memset(grad, 0, sizeof(double)*nx);
  

  space=(ub[0]-lb[0])/degree;

  for(i=0;i<degree;i++){
    t[0]=lb[0]+space*i;
    ival=gt(x, t, j, dlevel, gd, miu);
    if(ival>0){
      isgnval=1;
    } else {
      isgnval=0;
    }
    if(i==0 || i==degree-1){
      *integral+=ival/2;
      *isgn+=isgnval/2;
      for(k=0;k<nx;k++)
	grad[k]+=gd[k]/2;
    } else {
      *integral+=ival;
      *isgn+=isgnval;
      for(k=0;k<nx;k++)
	grad[k]+=gd[k];
    }
  }
  
  *integral *=space;
  *isgn *=space;

  printf("Trap %f\n", *integral);
#if !defined __linux__ && !defined linux
    free(gd);
#endif

}



/*******************************************************************
Evaluate the integral by a Newton-Cotes adaptative trapezoid formula
gt     - pointer to function
nt     - number of infinite variables
nx     - number of finite variables
lb     - array of lower bounds
ub     - array of upper bounds
x      - finite variables
t      - infinite variables (null initialized)
degree - number of start points in the formula
j      - number of infinite constraint
error  - error between refinements

return the integral value
************************************************************/
void adapt_trap(double (*gt) (), double *lb, double *ub, int nx, double *x,
		int nt, double *t, int degree, int j, double *integral,
		double *isgn, double error, double amp, int dlevel,
		double *grad, double miu)
{
  int i, k;
  double ival, isgnval, space;
  double llb, lub, flb, fub;

#if defined __linux__ || defined linux
  double gd[nx], gd1[nx], gd2[nx];
#else
  double *gd, *gd1, *gd2;
  gd=mymalloc(nx*sizeof(double));
  gd1=mymalloc(nx*sizeof(double));
  gd2=mymalloc(nx*sizeof(double));
#endif


  *integral=0.0;
  *isgn=0.0;
  if(dlevel==FIRST_DERIV)
    memset(grad, 0, sizeof(double)*nx);


  space=(ub[0]-lb[0])/degree;

  if(space==0.0)return;

  for(i=0;i<degree;i++){
    llb=lb[0]+space*i;
    lub=lb[0]+space*(i+1);
    flb=gt(x, &llb, j, dlevel, gd1, miu);
    fub=gt(x, &lub, j, dlevel, gd2, miu);
    ival=adapt_trap2(gt, llb, lub, flb, fub, nx, x, nt, t, j, error, amp,
		     &isgnval, dlevel, gd1, gd2, gd, miu);
    *integral+=ival;
    *isgn+=isgnval;
    if(dlevel==FIRST_DERIV){
      for(k=0;k<nx;k++)
	grad[k]+=gd[k];
    }
  }

  /*  printf("Adapt Trap %.12f\n", *integral);*/
  /*  fprintf(stderr,"Adapt Trap %f\n", *integral);*/

#if !defined __linux__ && !defined linux
    free(gd);
    free(gd1);
    free(gd2);
#endif

}


double adapt_trap2(double (*gt) (), double lb, double ub, double fl, double fu,
		   int nx, double *x, int nt, double *t, int j, double error,
		   double amp, double *isgnval, int dlevel, double *gl,
		   double *gu, double *grad, double miu)
{
  double I1, I2, fm, G1, G2;
  double isgnval1, isgnval2;
  double res, tm;
  int k;

#if defined __linux__ || defined linux
  double gd[nx], gd1[nx], gd2[nx];
#else
  double *gd, *gd1, *gd2;
  gd=mymalloc(nx*sizeof(double));
  gd1=mymalloc(nx*sizeof(double));
  gd2=mymalloc(nx*sizeof(double));
#endif
  

  if(lb>=ub){
    if(dlevel==FIRST_DERIV)
      memset(grad, 0, sizeof(double)*nx);
    return 0.0;
  }

  tm=(lb+ub)/2;
  fm=gt(x, &tm, j, dlevel, gd, miu);

  I1=(ub-lb)*(fl+fu)/2;
  I2=(ub-lb)*(fl+2*fm+fu)/4;
  G1=0.0;
  G2=0.0;
  if(dlevel==FIRST_DERIV){
    for(k=0;k<nx;k++)
      G1+=(ub-lb)*(gl[k]+gu[k])/2;
    for(k=0;k<nx;k++)
      G2+=(ub-lb)*(gl[k]+2*gd[k]+gu[k])/4;
  }


  if((ub-lb)<=amp || (fabs(I2-I1)<error)){ /*   && fabs(G2-G1)<error)){ */
    /*    printf("%f ", lb); 
	  fprintf(stderr, "%f ", f0);*/
    isgnval1=0;
    if(fl>0)
      isgnval1+=1;
    if(fm>0)
      isgnval1+=2;
    if(fu>0)
      isgnval1+=1;

    *isgnval=(ub-lb)*isgnval1/4;
    if(dlevel==FIRST_DERIV){
      /*        printf("tl=%10f\ttm=%10f\ttu=%10f\n", lb, tm, ub); */
      for(k=0;k<nx;k++){
	grad[k]=(ub-lb)*(gl[k]+2*gd[k]+gu[k])/4;
	/*  	printf("gl[%d]=%10f\tgm[%d]=%10f\tgu[%d]=%10f\n", k, gl[k], k,
		gd[k], k, gu[k]); */
      }
    }

#if !defined __linux__ && !defined linux
    free(gd);
    free(gd1);
    free(gd2);
#endif

    return I2;
  } else {
    /*    printf("Recursive called I1=%.12f, I2=%.12f\n", I1, I2);*/
    res=adapt_trap2(gt, lb, tm, fl, fm, nx, x, nt, t, j, error, amp,
		    &isgnval1, dlevel, gl, gd, gd1, miu);
    res+=adapt_trap2(gt, tm, ub, fm, fu, nx, x, nt, t, j, error, amp,
		     &isgnval2, dlevel, gd, gu, gd2, miu);
    *isgnval=isgnval1+isgnval2;
    if(dlevel==FIRST_DERIV){
      for(k=0;k<nx;k++)
	grad[k]=gd1[k]+gd2[k];
    }
#if !defined __linux__ && !defined linux
    free(gd);
    free(gd1);
    free(gd2);
#endif
    return res;
  }
#if !defined __linux__ && !defined linux
    free(gd);
    free(gd1);
    free(gd2);
#endif
}


