###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#  Robotics path planning problem
#   As described by Elke Haaren-Retagne, PhD Thesis,
#                 submited to Trier University, "A Semi-Infinite Programming
#                                   "Algorithm for Robot Trajectory Planning",
#                 1992
#   Originaly described by Kang G. Shin and Neil D. McKay
#
#   Example 2 (Trajectory 2) Quarter of a circle
#    Coded by A. Ismael F. Vaz 20/02/2001  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################
function bspline;
function dbspline;


param k;            # spline degree
param n;            # number of blend splines
# The number of knots can be changed, do not forget to change the Knots array
param nk;           # number of knots
param Knots{1..nk}; # knots
param epson;
param x0{1..n};     # starting guess
param t0;
param pi := 4*atan(1);

# Constants for the Bendix PACS robot
param Cgrav  :=9.81;
param Jt     :=12.3183;
param Koff   :=3.0;
param Mr     :=10.0;
param Mz     :=40.0;
param Kt     :=8.0;
param Kr     :=4.0;
param Kz     :=1.0;
param Usat1  :=2.0;
param Usat2  :=0.05;
param Usat3  :=2.0;
param Kgear1 :=0.01176;
param Kgear2 :=0.00318; 
param Kgear3 :=0.00318;
param Kmot1  :=0.0397;
param Kmot2  :=0.79557D-3;
param Kmot3  :=0.0397;
param Rmot1  :=1;
param Rmot2  :=1;
param Rmot3  :=1;
param Vmin1  :=-40;
param Vmin2  :=-40;
param Vmin3  :=-40;
param Vmax1  :=40;
param Vmax2  :=40;
param Vmax3  :=40;

var x{i in 1..n} := x0[i];  # x variables (coeficients in the spline)
var t := t0;                # spline parameter

# describe trajectory
var   zeta    = -t^2*(-2*t+3);
var   zetad1  = 6*t^2-6*t;    # first derivative
var   zetad2  = 12*t-6;       # second derivative
param zetad3 := 12;           # third derivative

# Quarter of a circle
var teta1  = (pi/4)-(pi/2)*zeta;
var teta1d1 = -(pi/2)*zetad1;
var teta1d2 = -(pi/2)*zetad2;
#var teta1d3 = -(pi/2)*zetad3;

param teta2  := 1;
param teta2d1 := 0;
param teta2d2 := 0;
#param teta2d3 := 0;

param teta3  := 1;
param teta3d1 := 0;
param teta3d2 := 0;
#param teta3d3 := 0;


var g =	   bspline(t,    k, n, {i in 1..n} x[i], {i in 1..nk} Knots[i]);
var gd1 = dbspline(t, 1, k, n, {i in 1..n} x[i], {i in 1..nk} Knots[i]);
var gd2 = dbspline(t, 2, k, n, {i in 1..n} x[i], {i in 1..nk} Knots[i]);

var phi1d1 = teta1d1/g;
var phi1d2 = (teta1d2-teta1d1*(gd1/g))/g^2;
var phi2d1 = teta2d1/g;
var phi2d2 = (teta2d2-teta2d1*(gd1/g))/g^2;
var phi3d1 = teta3d1/g;
var phi3d2 = (teta3d2-teta3d1*(gd1/g))/g^2;


var Ulow1 = (Kmot1/(Kgear1*Rmot1))*Vmin1-((Kmot1^2)/((Kgear1^2*Rmot1)))*phi1d1;
var Uupp1 = (Kmot1/(Kgear1*Rmot1))*Vmax1-((Kmot1^2)/((Kgear1^2*Rmot1)))*phi1d1;

var Ulow2 = (Kmot2/(Kgear2*Rmot2))*Vmin2-((Kmot2^2)/((Kgear2^2*Rmot2)))*phi2d1;
var Uupp2 = (Kmot2/(Kgear2*Rmot2))*Vmax2-((Kmot2^2)/((Kgear2^2*Rmot2)))*phi2d1;

var Ulow3 = (Kmot3/(Kgear3*Rmot3))*Vmin3-((Kmot3)^2/((Kgear3^2*Rmot3)))*phi3d1;
var Uupp3 = (Kmot3/(Kgear3*Rmot3))*Vmax3-((Kmot3)^2/((Kgear3^2*Rmot3)))*phi3d1;

var Umin1  = max(-(Usat1/Kgear1), Ulow1);
var Umax1  = min( (Usat1/Kgear1), Uupp1);

var Umin2  = max(-(Usat2/Kgear2), Ulow2);
var Umax2  = min( (Usat2/Kgear2), Uupp2);

var Umin3  = max(-(Usat3/Kgear3), Ulow3);
var Umax3  = min( (Usat3/Kgear3), Uupp3);

var U1     = (Jt-Koff*teta2+Mr*teta2^2)*phi1d2+2*(Mr*teta2-0.5*Koff)
            *phi1d1*phi2d1+Kt*phi1d1;

var U2     = Mr*phi2d2+(0.5*Koff-Mr*teta2)*phi1d1^2+Kr*phi2d1;

var U3     = Mz*phi3d2+Kz*phi3d1+Mz*Cgrav;


minimize obj:
	(1/k)*(sum {i in 1..n} (x[i]*(Knots[i+k]-Knots[i])));

subject to bound:
	0 <= t <= 0.999999;

subject to tpositive:
	-g <= -epson;

subject to tcons1i:
	Umin1-U1<=0;

subject to tcons1s:
	U1-Umax1<=0;

subject to tcons2i:
	Umin2-U2<=0;

subject to tcons2s:
	U2-Umax2<=0;

subject to tcons3i:
	Umin3-U3<=0;

subject to tcons3s:
	U3-Umax3<=0;


data;


param k  := 4;
param n  := 9;
param epson := 1D-2;
param nk := 13;
param Knots :=
	1	0
	2	0
	3	0
	4	0
	5	0.1666667
	6	0.3333333
	7	0.5
	8	0.6666667
	9	0.8333333
	10	1
	11	1
	12	1
	13	1;

param x0 := 
	1	1.0
	2	1.0
	3	1.0
	4	1.0
	5	1.0
	6	1.0
	7	1.0
	8	1.0
	9	1.0;



param t0 := 0;


# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
#printf "Original Solution\n";
#printf "obj=%lf\n", 0;

