###############################################################################
# Objective: Quadratic
# Constraints: Generic
###############################################################################
#  Optimal Signal Sets
#   As described by Mark S. Gockenbach and Anthony J. Kearsley,
#	"Optimal Signal Sets for Non-Gaussian Detectors",
#	 in SIAM Journal on Optimization, Vol. 9, No. 2, pp. 316-326
#	
#   Sine-Sine basis, Laplacian Noise
#    Coded by A. Ismael F. Vaz 10/06/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################
param pi := atan(1)*4;
param K;
param C;
param M;
param W1;
param W2;
param sigma;
param T:=50;
param x0{0..(M*K-1)};


var x {i in 0..(M*K-1)} := x0[i];
var t;
var d := 100;

###################
# Basis functions #
# sine-sine basis #
###################

# for infinite constraint
var phi0 = sqrt(2/T)*sin(2*pi*W1*t);
var phi1 = sqrt(2/T)*sin(2*pi*W2*t);
# for finite constraint
param sphi0 {l in 0..(T-1)} := sqrt(2/T)*sin(2*pi*W1*l/T);
param sphi1 {l in 0..(T-1)} := sqrt(2/T)*sin(2*pi*W2*l/T);

################################################
# Signals as a linear combination of the basis #
################################################
var signals {i in 1..M} = x[(i-1)*K]*phi0+x[(i-1)*K+1]*phi1;


###############
# The problem #
###############
minimize dp:
	-d^2;

subject to const1 {i in 1..(M-1), j in (i+1)..M}:
	-(1/T)*(
0.5*(abs((x[(i-1)*K]-x[(j-1)*K])*sphi0[0]+(x[(i-1)*K+1]-x[(j-1)*K+1])*sphi1[0])*sqrt(2)/sigma+exp(-abs((x[(i-1)*K]-x[(j-1)*K])*sphi0[0]+(x[(i-1)*K+1]-x[(j-1)*K+1])*sphi1[0])*sqrt(2)/sigma)-1)+
0.5*(abs((x[(i-1)*K]-x[(j-1)*K])*sphi0[T-1]+(x[(i-1)*K+1]-x[(j-1)*K+1])*sphi1[T-1])*sqrt(2)/sigma+exp(-abs((x[(i-1)*K]-x[(j-1)*K])*sphi0[T-1]+(x[(i-1)*K+1]-x[(j-1)*K+1])*sphi1[T-1])*sqrt(2)/sigma)-1)+
sum {l in 1..(T-2)} (abs((x[(i-1)*K]-x[(j-1)*K])*sphi0[l]+(x[(i-1)*K+1]-x[(j-1)*K+1])*sphi1[l])*sqrt(2)/sigma+exp(-abs((x[(i-1)*K]-x[(j-1)*K])*sphi0[l]+(x[(i-1)*K+1]-x[(j-1)*K+1])*sphi1[l])*sqrt(2)/sigma)-1)) <= -d^2;

subject to tconst2 {i in 1..M}:
	signals[i]^2-C <= 0;

subject to bound1:
	0<=t<=1;


################
# Problem data #
################
data;

param K:=2;
param C:=10; # c was sqrt(10) and C^2 in the constraint
param W1:=10;
param W2:=11;
param M:=16;
param sigma:=1;



# Generate a random initial guess
option randseed '';
for {i in 1..M} {
	let x0[(i-1)*K]:=30*(Uniform01()-0.5);
	let x0[(i-1)*K+1]:=30*(Uniform01()-0.5);
}



# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
#option reset_initial_guesses 1; # don't want a initial guess
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display dp;
display d;


############################################
# Help to draw the constellation in MATLAB #
# Just copy&paste to MATLAB command window #
############################################
for {i in 1..K} {
	printf "x%d=\[",i;
	for {j in 1..(M-1)} {
	printf "%f, ", x[(j-1)*K+(i-1)];
	};
	printf "%f\];\n", x[(M-1)*K+(i-1)];
};
printf "plot\(x1,x2,'o'\)\n";
