/********************************************************************
 ********************************************************************
  Interface for MATLAB and SIPAMPL

  A. Ismael F. Vaz
  Edite M. G. P. Fernandes
  M. Paula S. F. Gomes

  University of Minho, Portugal

  27/01/2000
  06/11/2002 Updated


  This files provides the following functions to MATLAB

         [x0,ntsipc,xbl,xbu] = sipampl('stub')
                       f = sipampl(x)
            [f,Grad] = sipampl(x)
[c, ceq, K1, ..., Kntsipc,s] = sipampl(x,s)
                               sipampl('msg',x)


*********************************************************************
*********************************************************************/
#include <sys/stat.h>
#include "mex.h"
#include "matrix.h"
#undef printf
#include "../../asl_pfgh.h"
#include "../sip.h"


static char msgbuf[256];
ASL *asl=NULL;

static void clean_house(void)
{
  if (asl)
    ASL_free(&asl);
  sip_free();
}


static void usage(void)
{
  mexErrMsgTxt("sipampl usage:\n\n"
	       "[x0,ntsipc,xbl,xbu]=sipampl('stub')\nor\n"
	       "f = sipampl(x)\nor\n"
	       "[f,Grad] = sipampl(x)\nor\n"
	       "[c, ceq, K1, ..., Kntsipc,s] = sipampl(x,s)\nor\n"
	       "sipampl('msg',x)\n");
}


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  FILE *nl;
  char stub[512];
  char stubnl[512];
  fint m, n, no, nz, mxr, mxc;
  real *X, *T, *RX, *RT, *nn, *C, *CEQ, **KR, *S;
  int i,j,k,eq,neq,ngrid,ngrid1,ngrid2;
  double step1, step2;
#if defined __linux__ || defined linux
  struct stat stat_file;
#else
  struct _stat stat_file;
#endif

  mexPrintf("mexFunction called with %d out and %d in\n", nlhs, nrhs);

  if (nrhs == 1) {
    if (nlhs == 4 && mxIsChar(prhs[0])){
      if(mxGetString(prhs[0], stub, sizeof(stub)))
        mexErrMsgTxt("First argument must be 'stub'\n");
      clean_house();

      /* Check if file 'stub.nl' exists */
      sprintf(stubnl, "%s.nl", stub);
#if defined __linux__ && defined linux
  if(stat(stubnl, &stat_file)){
#else
  if(_stat(stubnl, &stat_file)){
#endif
    sprintf(stubnl, "File %s.nl does not exist\n", stub);
    mexErrMsgTxt(stubnl);
  }
      
      asl = ASL_alloc(ASL_read_pfgh);
      want_xpi0=1;

      mexAtExit(clean_house);
      
      nl=jac_dim_ASL(asl, stub, &m, &n, &no, &nz, &mxr, &mxc, (fint)strlen(stub));
      if (!nl)
        mexErrMsgTxt("Can't read problem\n");
      
      pfgh_read_ASL(asl, nl, 0);
      
      
      if (mxr == 0 || mxc == 0)
        mxErrMsgTxt("Semi-Infinite Programming interface needs .row and .col files.\n"
            "In order to provide .row and .col files, in your AMPL session say\n"
            "\toption auxfiles rc;\nbefore saying\n\twrite ...\n");
            
      sip_init(asl);

      if(ntsip<1 || ntsip>2)
        mxErrMsgTxt("Number of semi-infinite variables must be positive and less or equal to 2\n");
      if(ntsipc<1)
        mxErrMsgTxt("Number of semi-infinite constraints must be greater then zero.\n"
        "Semi-Infinite Programming interface needs .row and .col files.\n"
        "In order to provide .row and .col files, in your AMPL session say\n"
        "\toption auxfiles rc;\nbefore saying\n\twrite ...\n");
      
      
      if(X0 && (!havex0 || (havex0 && *havex0))){
        X=mxGetPr(plhs[0]=mxCreateDoubleMatrix(nxsip, 1, mxREAL));
        sip_extractx(X0, X);
      }
      
      nn=mxGetPr(plhs[1]=mxCreateDoubleMatrix(1, 1, mxREAL));
      nn[0]=(double)ntsipc;
      
      nn=mxGetPr(plhs[2]=mxCreateDoubleMatrix(nxsip, 1, mxREAL));
      memcpy(nn,XBL,nxsip*sizeof(double));
      
      nn=mxGetPr(plhs[3]=mxCreateDoubleMatrix(nxsip, 1, mxREAL));
      memcpy(nn,XBU,nxsip*sizeof(double));
      
      return;
    }
  }

  if(!asl)
    mxErrMsgTxt("You must call\n\t[x0,ntsipc,xbl,xbu]=sipampl(stub)\n first");


  if(nlhs<=2 && nrhs==1 && mxIsDouble(prhs[0])){
    X=mxGetPr(prhs[0]);
    nn=mxGetPr(plhs[0]=mxCreateDoubleMatrix(1, 1, mxREAL));
    nn[0]=sip_objval(0, X, NULL);
    if(nlhs==2){
      RX=mxGetPr(plhs[1]=mxCreateDoubleMatrix(nxsip, 1, mxREAL));
      sip_objgrd(0,X, RX, NULL);
    }
    return;
  }

  
  if(nrhs==2){ /* all constraints values */
    if(nlhs==0 && mxIsChar(prhs[0]) && mxIsDouble(prhs[1])){
      X=mxGetPr(prhs[1]);
      if(mxGetString(prhs[0], msgbuf, sizeof(msgbuf)))
        usage();
      write_sol(msgbuf, X, NULL, NULL);
      return;
    }
    
    if(nlhs!=3+ntsipc || !mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]))
      usage();

    if(ntsip>2)
      mxErrMsgTxt("Number of infinite variables too big (max=2)");

    /* count how many equality constraints */
    for(i=0,neq=0;i<nxsipc;i++)
      if(XCBL[i]==XCBU[i])neq++;

    C=mxGetPr(plhs[0]=mxCreateDoubleMatrix(nxsipc-neq, 1, mxREAL));
    CEQ=mxGetPr(plhs[1]=mxCreateDoubleMatrix(neq, 1, mxREAL));


    RX=mymalloc(nxsipc*sizeof(double));
    RT=mymalloc(ntsipc*sizeof(double));
    T=mymalloc(ntsip*sizeof(double));
    KR=mymalloc(ntsipc*sizeof(double *));

    X=mxGetPr(prhs[0]);
    S=mxGetPr(prhs[1]);

    if(ntsip==1){
      if(mxIsNaN(*S)){
        step1=0.1;
      } else {
        step1=*S;
      }

      if(step1<=0 || step1>(*TBU-*TBL))
        mexErrMsgTxt("Sample interval invalid");

      ngrid=(*TBU-*TBL)/(step1);

      for(i=0;i<ntsipc;i++)
        KR[i]=mxGetPr(plhs[2+i]=mxCreateDoubleMatrix(ngrid, ntsip, mxREAL));

      for(*T=*TBL,j=0;*T<=*TBU && j<ngrid;*T+=step1,j++){
        sip_conval(X, T, RX, RT, NULL);
        for(i=0;i<ntsipc;i++)
          *(KR[i]+j)=RT[i]-TCBU[i]; /* Only lower or equal infinite constraints */
      }
      for(i=0,neq=0,eq=0;i<nxsipc;i++){
        if(XCBL[i]==XCBU[i]){
            C[neq]=RX[i];
            neq++;
        } else {
            CEQ[eq]=RX[i];
            eq++;
        }
      }

      nn=mxGetPr(plhs[2+ntsipc]=mxCreateDoubleMatrix(ntsipc, 2, mxREAL)); /* aivaz */
      for(i=0;i<ntsipc;i++){
        nn[2*i]=step1;
        nn[2*i+1]=0;
      }
    } else {
      if(mxIsNaN(*S) || mxIsNaN(S[1])){
        step1=0.1;
        step2=0.1;
        } else {
        step1=S[0];
        step2=S[1];
      }
      
      if(step1<=0 || step1>(TBU[0]-TBL[0]))
        mexErrMsgTxt("Sample interval invalid");
      if(step2<=0 || step2>(TBU[1]-TBL[1]))
        mexErrMsgTxt("Sample interval invalid");

      ngrid1=(TBU[0]-TBL[0])/(step1);
      ngrid2=(TBU[1]-TBL[1])/(step2);

      for(i=0;i<ntsipc;i++)
        KR[i]=mxGetPr(plhs[2+i]=mxCreateDoubleMatrix(ngrid1, ngrid2, mxREAL));

    for(T[0]=TBL[0],j=0;T[0]<TBU[0] && j<ngrid1;T[0]+=step1,j++){
        for(T[1]=TBL[1],k=0;T[1]<TBU[1] && k<ngrid2;T[1]+=step2,k++){
            sip_conval(X, T, RX, RT, NULL);
            for(i=0;i<ntsipc;i++){
                *(KR[i]+j*ngrid2+k)=RT[i]-TCBU[i];
            }
        }
    }

    for(i=0,neq=0,eq=0;i<nxsipc;i++){
        if(XCBL[i]==XCBU[i]){
            C[neq]=RX[i];
            neq++;
        } else {
            CEQ[eq]=RX[i];
            eq++;
        }
    }
    nn=mxGetPr(plhs[2+ntsipc]=mxCreateDoubleMatrix(ntsipc, 2, mxREAL));
    for(i=0;i<ntsipc;i++){
        nn[2*i]=step1;
        nn[2*i+1]=step2;
      }
  }
  free(RX);
  free(RT);
  free(T);
  free(KR);

  return;
  }

  mexPrintf("Invalid number of input/output arguments\n");
  
  usage();
    
}

#if defined __linux__ || defined linux
void __sigsetjmp (sigjmp_buf env, int savemask)
{
}
#endif
